@@ -6,11 +6,14 @@
 SSH key exchange handling.
 """
 
-from hashlib import sha1, sha256
+from __future__ import absolute_import, division
+
+from hashlib import sha1, sha256, sha384, sha512
 
 from zope.interface import Attribute, implementer, Interface
 
 from twisted.conch import error
+from twisted.python.compat import long
 
 
 class _IKexAlgorithm(Interface):
@@ -19,7 +22,7 @@ class _IKexAlgorithm(Interface):
     """
 
     preference = Attribute(
-        "An C{int} giving the preference of the algorithm when negotiating "
+        "An L{int} giving the preference of the algorithm when negotiating "
         "key exchange. Algorithms with lower precedence values are more "
         "preferred.")
 
@@ -36,16 +39,24 @@ class _IFixedGroupKexAlgorithm(_IKexAlgorithm):
     """
 
     prime = Attribute(
-        "A C{long} giving the prime number used in Diffie-Hellman key "
-        "exchange, or C{None} if not applicable.")
+        "A L{long} giving the prime number used in Diffie-Hellman key "
+        "exchange, or L{None} if not applicable.")
 
     generator = Attribute(
-        "A C{long} giving the generator number used in Diffie-Hellman key "
-        "exchange, or C{None} if not applicable. (This is not related to "
+        "A L{long} giving the generator number used in Diffie-Hellman key "
+        "exchange, or L{None} if not applicable. (This is not related to "
         "Python generator functions.)")
 
 
 
+class _IEllipticCurveExchangeKexAlgorithm(_IKexAlgorithm):
+    """
+    An L{_IEllipticCurveExchangeKexAlgorithm} describes a key exchange algorithm
+    that uses an elliptic curve exchange between the client and server.
+    """
+
+
+
 class _IGroupExchangeKexAlgorithm(_IKexAlgorithm):
     """
     An L{_IGroupExchangeKexAlgorithm} describes a key exchange algorithm
@@ -57,6 +68,39 @@ class _IGroupExchangeKexAlgorithm(_IKexAlgorithm):
 
 
 
+@implementer(_IEllipticCurveExchangeKexAlgorithm)
+class _ECDH256(object):
+    """
+    Elliptic Curve Key Exchange with SHA-256 as HASH. Defined in
+    RFC 5656.
+    """
+    preference = 1
+    hashProcessor = sha256
+
+
+
+@implementer(_IEllipticCurveExchangeKexAlgorithm)
+class _ECDH384(object):
+    """
+    Elliptic Curve Key Exchange with SHA-384 as HASH. Defined in
+    RFC 5656.
+    """
+    preference = 2
+    hashProcessor = sha384
+
+
+
+@implementer(_IEllipticCurveExchangeKexAlgorithm)
+class _ECDH512(object):
+    """
+    Elliptic Curve Key Exchange with SHA-512 as HASH. Defined in
+    RFC 5656.
+    """
+    preference = 3
+    hashProcessor = sha512
+
+
+
 @implementer(_IGroupExchangeKexAlgorithm)
 class _DHGroupExchangeSHA256(object):
     """
@@ -64,7 +108,7 @@ class _DHGroupExchangeSHA256(object):
     RFC 4419, 4.2.
     """
 
-    preference = 1
+    preference = 4
     hashProcessor = sha256
 
 
@@ -76,7 +120,7 @@ class _DHGroupExchangeSHA1(object):
     RFC 4419, 4.1.
     """
 
-    preference = 2
+    preference = 5
     hashProcessor = sha1
 
 
@@ -88,14 +132,14 @@ class _DHGroup1SHA1(object):
     (1024-bit MODP Group). Defined in RFC 4253, 8.1.
     """
 
-    preference = 3
+    preference = 6
     hashProcessor = sha1
     # Diffie-Hellman primes from Oakley Group 2 (RFC 2409, 6.2).
     prime = long('17976931348623159077083915679378745319786029604875601170644'
         '44236841971802161585193689478337958649255415021805654859805036464405'
         '48199239100050792877003355816639229553136239076508735759914822574862'
         '57500742530207744771258955095793777842444242661733472762929938766870'
-        '9205606050270810842907692932019128194467627007L')
+        '9205606050270810842907692932019128194467627007')
     generator = 2
 
 
@@ -107,7 +151,7 @@ class _DHGroup14SHA1(object):
     (2048-bit MODP Group). Defined in RFC 4253, 8.2.
     """
 
-    preference = 4
+    preference = 7
     hashProcessor = sha1
     # Diffie-Hellman primes from Oakley Group 14 (RFC 3526, 3).
     prime = long('32317006071311007300338913926423828248817941241140239112842'
@@ -119,16 +163,28 @@ class _DHGroup14SHA1(object):
         '00977202194168647225871031411336429319536193471636533209717077448227'
         '98858856536920864529663607725026895550592836275112117409697299806841'
         '05543595848665832916421362182310789909994486524682624169720359118525'
-        '07045361090559L')
+        '07045361090559')
     generator = 2
 
 
 
+# Which ECDH hash function to use is dependent on the size.
 _kexAlgorithms = {
-    "diffie-hellman-group-exchange-sha256": _DHGroupExchangeSHA256(),
-    "diffie-hellman-group-exchange-sha1": _DHGroupExchangeSHA1(),
-    "diffie-hellman-group1-sha1": _DHGroup1SHA1(),
-    "diffie-hellman-group14-sha1": _DHGroup14SHA1(),
+    b"diffie-hellman-group-exchange-sha256": _DHGroupExchangeSHA256(),
+    b"diffie-hellman-group-exchange-sha1": _DHGroupExchangeSHA1(),
+    b"diffie-hellman-group1-sha1": _DHGroup1SHA1(),
+    b"diffie-hellman-group14-sha1": _DHGroup14SHA1(),
+    b"ecdh-sha2-nistp256": _ECDH256(),
+    b"ecdh-sha2-nistp384": _ECDH384(),
+    b"ecdh-sha2-nistp521": _ECDH512(),
+    b"ecdh-sha2-nistk163": _ECDH256(),
+    b"ecdh-sha2-nistp224": _ECDH256(),
+    b"ecdh-sha2-nistk233": _ECDH256(),
+    b"ecdh-sha2-nistb233": _ECDH256(),
+    b"ecdh-sha2-nistk283": _ECDH384(),
+    b"ecdh-sha2-nistk409": _ECDH384(),
+    b"ecdh-sha2-nistb409": _ECDH384(),
+    b"ecdh-sha2-nistt571": _ECDH512()
     }
 
 
@@ -138,7 +194,7 @@ def getKex(kexAlgorithm):
     Get a description of a named key exchange algorithm.
 
     @param kexAlgorithm: The key exchange algorithm name.
-    @type kexAlgorithm: C{str}
+    @type kexAlgorithm: L{bytes}
 
     @return: A description of the key exchange algorithm named by
         C{kexAlgorithm}.
@@ -153,16 +209,31 @@ def getKex(kexAlgorithm):
 
 
 
+def isEllipticCurve(kexAlgorithm):
+    """
+    Returns C{True} if C{kexAlgorithm} is an elliptic curve.
+
+    @param kexAlgorithm: The key exchange algorithm name.
+    @type kexAlgorithm: C{str}
+
+    @return: C{True} if C{kexAlgorithm} is an elliptic curve,
+        otherwise C{False}.
+    @rtype: C{bool}
+    """
+    return _IEllipticCurveExchangeKexAlgorithm.providedBy(getKex(kexAlgorithm))
+
+
+
 def isFixedGroup(kexAlgorithm):
     """
     Returns C{True} if C{kexAlgorithm} has a fixed prime / generator group.
 
     @param kexAlgorithm: The key exchange algorithm name.
-    @type kexAlgorithm: C{str}
+    @type kexAlgorithm: L{bytes}
 
     @return: C{True} if C{kexAlgorithm} has a fixed prime / generator group,
         otherwise C{False}.
-    @rtype: C{bool}
+    @rtype: L{bool}
     """
     return _IFixedGroupKexAlgorithm.providedBy(getKex(kexAlgorithm))
 
@@ -173,7 +244,7 @@ def getHashProcessor(kexAlgorithm):
     Get the hash algorithm callable to use in key exchange.
 
     @param kexAlgorithm: The key exchange algorithm name.
-    @type kexAlgorithm: C{str}
+    @type kexAlgorithm: L{bytes}
 
     @return: A callable hash algorithm constructor (e.g. C{hashlib.sha256}).
     @rtype: C{callable}
@@ -188,10 +259,10 @@ def getDHGeneratorAndPrime(kexAlgorithm):
     Get the generator and the prime to use in key exchange.
 
     @param kexAlgorithm: The key exchange algorithm name.
-    @type kexAlgorithm: C{str}
+    @type kexAlgorithm: L{bytes}
 
-    @return: A C{tuple} containing C{long} generator and C{long} prime.
-    @rtype: C{tuple}
+    @return: A L{tuple} containing L{long} generator and L{long} prime.
+    @rtype: L{tuple}
     """
     kex = getKex(kexAlgorithm)
     return kex.generator, kex.prime
@@ -204,8 +275,21 @@ def getSupportedKeyExchanges():
     preference.
 
     @return: A C{list} of supported key exchange algorithm names.
-    @rtype: C{list} of C{str}
+    @rtype: C{list} of L{bytes}
     """
+    from cryptography.hazmat.backends import default_backend
+    from cryptography.hazmat.primitives.asymmetric import ec
+    from twisted.conch.ssh.keys import _curveTable
+
+    backend = default_backend()
+    kexAlgorithms = _kexAlgorithms.copy()
+    for keyAlgorithm in list(kexAlgorithms):
+        if keyAlgorithm.startswith(b"ecdh"):
+            keyAlgorithmDsa = keyAlgorithm.replace(b"ecdh", b"ecdsa")
+            supported = backend.elliptic_curve_exchange_algorithm_supported(
+                ec.ECDH(), _curveTable[keyAlgorithmDsa])
+            if not supported:
+                kexAlgorithms.pop(keyAlgorithm)
     return sorted(
-        _kexAlgorithms,
-        key = lambda kexAlgorithm: _kexAlgorithms[kexAlgorithm].preference)
+        kexAlgorithms,
+        key = lambda kexAlgorithm: kexAlgorithms[kexAlgorithm].preference)
