@@ -24,27 +24,34 @@ if pyasn1 is not None and cryptography is not None:
     dependencySkip = None
     from twisted.conch.ssh import transport, keys, factory
     from twisted.conch.test import keydata
+    from cryptography.hazmat.backends import default_backend
+    from cryptography.hazmat.primitives.asymmetric import ec
+    from cryptography.exceptions import UnsupportedAlgorithm
 else:
     if pyasn1 is None:
         dependencySkip = "Cannot run without PyASN1"
     elif cryptography is None:
         dependencySkip = "can't run without cryptography"
 
+
     class transport: # fictional modules to make classes work
         class SSHTransportBase: pass
         class SSHServerTransport: pass
         class SSHClientTransport: pass
+
+
     class factory:
         class SSHFactory:
             pass
 
-from hashlib import md5, sha1, sha256, sha512
+from hashlib import md5, sha1, sha256, sha384, sha512
 
 from twisted.trial import unittest
 from twisted.internet import defer
 from twisted.protocols import loopback
 from twisted.python import randbytes
 from twisted.python.randbytes import insecureRandom
+from twisted.python.compat import iterbytes, _bytesChr as chr
 from twisted.conch.ssh import address, service, common, _kex
 from twisted.test import proto_helpers
 
@@ -78,7 +85,7 @@ class MockTransportBase(transport.SSHTransportBase):
         """
         Intercept unsupported version call.
 
-        @type remoteVersion: C{str}
+        @type remoteVersion: L{str}
         """
         self.gotUnsupportedVersion = remoteVersion
         return transport.SSHTransportBase._unsupportedVersionReceived(
@@ -89,8 +96,8 @@ class MockTransportBase(transport.SSHTransportBase):
         """
         Store any errors received.
 
-        @type reasonCode: C{int}
-        @type description: C{str}
+        @type reasonCode: L{int}
+        @type description: L{str}
         """
         self.errors.append((reasonCode, description))
 
@@ -99,7 +106,7 @@ class MockTransportBase(transport.SSHTransportBase):
         """
         Store any unimplemented packet messages.
 
-        @type seqnum: C{int}
+        @type seqnum: L{int}
         """
         self.unimplementeds.append(seqnum)
 
@@ -108,9 +115,9 @@ class MockTransportBase(transport.SSHTransportBase):
         """
         Store any debug messages.
 
-        @type alwaysDisplay: C{bool}
-        @type message: C{str}
-        @type lang: C{str}
+        @type alwaysDisplay: L{bool}
+        @type message: L{str}
+        @type lang: L{str}
         """
         self.debugs.append((alwaysDisplay, message, lang))
 
@@ -119,7 +126,7 @@ class MockTransportBase(transport.SSHTransportBase):
         """
         Store any ignored data.
 
-        @type packet: C{str}
+        @type packet: L{str}
         """
         self.ignoreds.append(packet)
 
@@ -217,7 +224,7 @@ class MockService(service.SSHService):
     @ivar started: True if this service has been started.
     @ivar stopped: True if this service has been stopped.
     """
-    name = "MockService"
+    name = b"MockService"
     started = False
     stopped = False
     protocolMessages = {0xff: "MSG_TEST", 71: "MSG_fiction"}
@@ -254,7 +261,7 @@ class MockFactory(factory.SSHFactory):
     A mocked-up factory based on twisted.conch.ssh.factory.SSHFactory.
     """
     services = {
-        'ssh-userauth': MockService}
+        b'ssh-userauth': MockService}
 
 
     def getPublicKeys(self):
@@ -281,7 +288,7 @@ class MockFactory(factory.SSHFactory):
         that use group exchange to establish a prime / generator group.
 
         @return: The primes and generators.
-        @rtype: C{dict} mapping the key size to a C{list} of
+        @rtype: L{dict} mapping the key size to a C{list} of
             C{(generator, prime)} tuple.
         """
         # In these tests, we hardwire the prime values to those defined by the
@@ -409,6 +416,16 @@ class DHGroupExchangeSHA256Mixin:
 
 
 
+class ECDHMixin:
+    """
+    Mixin for elliptic curve diffie-hellman tests.
+    """
+
+    kexAlgorithm = b'ecdh-sha2-nistp256'
+    hashProcessor = sha256
+
+
+
 class BaseSSHTransportBaseCase:
     """
     Base case for TransportBase tests.
@@ -462,7 +479,7 @@ class BaseSSHTransportTests(BaseSSHTransportBaseCase, TransportTestCase):
         proto.makeConnection(self.transport)
         self.finishKeyExchange(proto)
         self.transport.clear()
-        message = ord(b'A')
+        message = ord('A')
         payload = b'BCDEFG'
         proto.sendPacket(message, payload)
         value = self.transport.value()
@@ -574,7 +591,7 @@ class BaseSSHTransportTests(BaseSSHTransportBaseCase, TransportTestCase):
         proto.sendPacket(ord('A'), b'BCD')
         value = self.transport.value()
         proto.buf = value[:MockCipher.decBlockSize]
-        self.assertEqual(proto.getPacket(), None)
+        self.assertIsNone(proto.getPacket())
         self.assertTrue(testCipher.usedDecrypt)
         self.assertEqual(proto.first, b'\x00\x00\x00\x0e\x09A')
         proto.buf += value[MockCipher.decBlockSize:]
@@ -939,7 +956,7 @@ class BaseSSHTransportTests(BaseSSHTransportBaseCase, TransportTestCase):
         self.transport.loseConnection = stubLoseConnection
         self.proto.loseConnection()
         self.assertEqual(self.packets[0][0], transport.MSG_DISCONNECT)
-        self.assertEqual(self.packets[0][1][3],
+        self.assertEqual(self.packets[0][1][3:4],
                          chr(transport.DISCONNECT_CONNECTION_LOST))
 
 
@@ -954,12 +971,12 @@ class BaseSSHTransportTests(BaseSSHTransportBaseCase, TransportTestCase):
             def stubLoseConnection():
                 disconnected[0] = True
             self.transport.loseConnection = stubLoseConnection
-            for c in version + b'\r\n':
+            for c in iterbytes(version + b'\r\n'):
                 self.proto.dataReceived(c)
             self.assertTrue(disconnected[0])
             self.assertEqual(self.packets[0][0], transport.MSG_DISCONNECT)
             self.assertEqual(
-                self.packets[0][1][3],
+                self.packets[0][1][3:4],
                 chr(transport.DISCONNECT_PROTOCOL_VERSION_NOT_SUPPORTED))
         testBad(b'SSH-1.5-OpenSSH')
         testBad(b'SSH-3.0-Twisted')
@@ -975,14 +992,14 @@ class BaseSSHTransportTests(BaseSSHTransportBaseCase, TransportTestCase):
         data = (b"""here's some stuff beforehand
 here's some other stuff
 """ + proto.ourVersionString + b"\r\n")
-        [proto.dataReceived(c) for c in data]
+        [proto.dataReceived(c) for c in iterbytes(data)]
         self.assertTrue(proto.gotVersion)
         self.assertEqual(proto.otherVersionString, proto.ourVersionString)
 
 
     def test_compatabilityVersion(self):
         """
-        Test that the transport treats the compatbility version (1.99)
+        Test that the transport treats the compatibility version (1.99)
         as equivalent to version 2.0.
         """
         proto = MockTransportBase()
@@ -1024,10 +1041,10 @@ here's some other stuff
         def testBad(packet, error=transport.DISCONNECT_PROTOCOL_ERROR):
             self.packets = []
             self.proto.buf = packet
-            self.assertEqual(self.proto.getPacket(), None)
+            self.assertIsNone(self.proto.getPacket())
             self.assertEqual(len(self.packets), 1)
             self.assertEqual(self.packets[0][0], transport.MSG_DISCONNECT)
-            self.assertEqual(self.packets[0][1][3], chr(error))
+            self.assertEqual(self.packets[0][1][3:4], chr(error))
 
         testBad(b'\xff' * 8) # big packet
         testBad(b'\x00\x00\x00\x05\x00BCDE') # length not modulo blocksize
@@ -1056,7 +1073,7 @@ here's some other stuff
         def checkUnimplemented(seqnum=seqnum):
             self.assertEqual(self.packets[0][0],
                              transport.MSG_UNIMPLEMENTED)
-            self.assertEqual(self.packets[0][1][3], chr(seqnum))
+            self.assertEqual(self.packets[0][1][3:4], chr(seqnum))
             self.proto.packets = []
             seqnum += 1
 
@@ -1136,6 +1153,15 @@ class BaseSSHTransportDHGroupExchangeSHA256Tests(
 
 
 
+class BaseSSHTransportEllipticCurveTests(
+        BaseSSHTransportDHGroupExchangeBaseCase, ECDHMixin,
+        TransportTestCase):
+    """
+    ecdh-sha2-nistp256 tests for TransportBase
+    """
+
+
+
 class ServerAndClientSSHTransportBaseCase:
     """
     Tests that need to be run on both the server and the client.
@@ -1148,7 +1174,7 @@ class ServerAndClientSSHTransportBaseCase:
         if kind is None:
             kind = transport.DISCONNECT_PROTOCOL_ERROR
         self.assertEqual(self.packets[-1][0], transport.MSG_DISCONNECT)
-        self.assertEqual(self.packets[-1][1][3], chr(kind))
+        self.assertEqual(self.packets[-1][1][3:4], chr(kind))
 
 
     def connectModifiedProtocol(self, protoModification,
@@ -1376,7 +1402,7 @@ class ServerSSHTransportTests(ServerSSHTransportBaseCase, TransportTestCase):
         KEXDH_REPLY with the server's public key and a signature.
 
         @param kexAlgorithm: The key exchange algorithm to use.
-        @type kexAlgorithm: C{str}
+        @type kexAlgorithm: L{str}
         """
         self.proto.supportedKeyExchanges = [kexAlgorithm]
         self.proto.supportedPublicKeys = [b'ssh-rsa']
@@ -1410,6 +1436,42 @@ class ServerSSHTransportTests(ServerSSHTransportBaseCase, TransportTestCase):
              (transport.MSG_NEWKEYS, b'')])
 
 
+    def test_checkBad_KEX_ECDH_INIT_CurveName(self):
+        """
+        Test that if the server receives a KEX_DH_GEX_REQUEST_OLD message
+        and the key exchange algorithm is not set, we raise a ConchError.
+        """
+        self.proto.kexAlg = b'bad-curve'
+        self.proto.keyAlg = b'ssh-rsa'
+        self.assertRaises(UnsupportedAlgorithm,
+                          self.proto._ssh_KEX_ECDH_INIT,
+                          common.NS(b'unused-key'))
+
+
+    def test_checkBad_KEX_INIT_CurveName(self):
+        """
+        Test that if the server received a bad name for a curve
+        we raise an UnsupportedAlgorithm error.
+        """
+        kexmsg = (
+            b"\xAA" * 16 +
+            common.NS(b'ecdh-sha2-nistp256') +
+            common.NS(b'ssh-rsa') +
+            common.NS(b'aes256-ctr') +
+            common.NS(b'aes256-ctr') +
+            common.NS(b'hmac-sha1') +
+            common.NS(b'hmac-sha1') +
+            common.NS(b'none') +
+            common.NS(b'none') +
+            common.NS(b'') +
+            common.NS(b'') +
+            b'\x00' + b'\x00\x00\x00\x00')
+
+        self.proto.ssh_KEXINIT(kexmsg)
+        self.assertRaises(AttributeError)
+        self.assertRaises(UnsupportedAlgorithm)
+
+
     def test_KEXDH_INIT_GROUP1(self):
         """
         KEXDH_INIT messages are processed when the
@@ -1438,7 +1500,26 @@ class ServerSSHTransportTests(ServerSSHTransportBaseCase, TransportTestCase):
         self.assertEqual(self.proto.sessionID, b'CD')
         self.assertEqual(self.packets[-1], (transport.MSG_NEWKEYS, b''))
         newKeys = [self.proto._getKey(c, b'AB', b'EF')
-                   for c in b'ABCDEF']
+                   for c in iterbytes(b'ABCDEF')]
+        self.assertEqual(
+            self.proto.nextEncryptions.keys,
+            (newKeys[1], newKeys[3], newKeys[0], newKeys[2], newKeys[5],
+             newKeys[4]))
+
+
+    def test_ECDH_keySetup(self):
+        """
+        Test that _keySetup sets up the next encryption keys.
+        """
+        self.proto.kexAlg = b'ecdh-sha2-nistp256'
+        self.proto.nextEncryptions = MockCipher()
+        self.simulateKeyExchange(b'AB', b'CD')
+        self.assertEqual(self.proto.sessionID, b'CD')
+        self.simulateKeyExchange(b'AB', b'EF')
+        self.assertEqual(self.proto.sessionID, b'CD')
+        self.assertEqual(self.packets[-1], (transport.MSG_NEWKEYS, b''))
+        newKeys = [self.proto._getKey(c, b'AB', b'EF')
+                   for c in iterbytes(b'ABCDEF')]
         self.assertEqual(
             self.proto.nextEncryptions.keys,
             (newKeys[1], newKeys[3], newKeys[0], newKeys[2], newKeys[5],
@@ -1457,16 +1538,16 @@ class ServerSSHTransportTests(ServerSSHTransportBaseCase, TransportTestCase):
         self.proto.ssh_NEWKEYS(b'')
         self.assertIs(self.proto.currentEncryptions,
                       self.proto.nextEncryptions)
-        self.assertIs(self.proto.outgoingCompression, None)
-        self.assertIs(self.proto.incomingCompression, None)
+        self.assertIsNone(self.proto.outgoingCompression)
+        self.assertIsNone(self.proto.incomingCompression)
         self.proto.outgoingCompressionType = b'zlib'
         self.simulateKeyExchange(b'AB', b'CD')
         self.proto.ssh_NEWKEYS(b'')
-        self.assertIsNot(self.proto.outgoingCompression, None)
+        self.assertIsNotNone(self.proto.outgoingCompression)
         self.proto.incomingCompressionType = b'zlib'
         self.simulateKeyExchange(b'AB', b'EF')
         self.proto.ssh_NEWKEYS(b'')
-        self.assertIsNot(self.proto.incomingCompression, None)
+        self.assertIsNotNone(self.proto.incomingCompression)
 
 
     def test_SERVICE_REQUEST(self):
@@ -1477,7 +1558,7 @@ class ServerSSHTransportTests(ServerSSHTransportBaseCase, TransportTestCase):
         self.proto.ssh_SERVICE_REQUEST(common.NS(b'ssh-userauth'))
         self.assertEqual(self.packets, [(transport.MSG_SERVICE_ACCEPT,
                                          common.NS(b'ssh-userauth'))])
-        self.assertEqual(self.proto.service.name, 'MockService')
+        self.assertEqual(self.proto.service.name, b'MockService')
 
 
     def test_disconnectNEWKEYSData(self):
@@ -1724,7 +1805,7 @@ class ClientSSHTransportTests(ClientSSHTransportBaseCase, TransportTestCase):
         results in a correct KEXDH_INIT response.
 
         @param kexAlgorithm: The key exchange algorithm to use
-        @type kexAlgorithm: C{str}
+        @type kexAlgorithm: L{str}
         """
         self.proto.supportedKeyExchanges = [kexAlgorithm]
 
@@ -1785,8 +1866,8 @@ class ClientSSHTransportTests(ClientSSHTransportBaseCase, TransportTestCase):
         exchangeHash = h.digest()
 
         def _cbTestKEXDH_REPLY(value):
-            self.assertIs(value, None)
-            self.assertEqual(self.calledVerifyHostKey, True)
+            self.assertIsNone(value)
+            self.assertTrue(self.calledVerifyHostKey)
             self.assertEqual(self.proto.sessionID, exchangeHash)
 
         signature = self.privObj.sign(exchangeHash)
@@ -1810,7 +1891,8 @@ class ClientSSHTransportTests(ClientSSHTransportBaseCase, TransportTestCase):
         self.simulateKeyExchange(b'AB', b'EF')
         self.assertEqual(self.proto.sessionID, b'CD')
         self.assertEqual(self.packets[-1], (transport.MSG_NEWKEYS, b''))
-        newKeys = [self.proto._getKey(c, b'AB', b'EF') for c in b'ABCDEF']
+        newKeys = [self.proto._getKey(c, b'AB', b'EF')
+                   for c in iterbytes(b'ABCDEF')]
         self.assertEqual(self.proto.nextEncryptions.keys,
                           (newKeys[0], newKeys[2], newKeys[1], newKeys[3],
                            newKeys[4], newKeys[5]))
@@ -1835,19 +1917,19 @@ class ClientSSHTransportTests(ClientSSHTransportBaseCase, TransportTestCase):
 
         self.proto.nextEncryptions = MockCipher()
         self.proto.ssh_NEWKEYS(b'')
-        self.assertIs(self.proto.outgoingCompression, None)
-        self.assertIs(self.proto.incomingCompression, None)
+        self.assertIsNone(self.proto.outgoingCompression)
+        self.assertIsNone(self.proto.incomingCompression)
         self.assertIs(self.proto.currentEncryptions,
                       self.proto.nextEncryptions)
         self.assertTrue(secure[0])
         self.proto.outgoingCompressionType = b'zlib'
         self.simulateKeyExchange(b'AB', b'GH')
         self.proto.ssh_NEWKEYS(b'')
-        self.assertIsNot(self.proto.outgoingCompression, None)
+        self.assertIsNotNone(self.proto.outgoingCompression)
         self.proto.incomingCompressionType = b'zlib'
         self.simulateKeyExchange(b'AB', b'IJ')
         self.proto.ssh_NEWKEYS(b'')
-        self.assertIsNot(self.proto.incomingCompression, None)
+        self.assertIsNotNone(self.proto.incomingCompression)
 
 
     def test_SERVICE_ACCEPT(self):
@@ -1877,6 +1959,48 @@ class ClientSSHTransportTests(ClientSSHTransportBaseCase, TransportTestCase):
         self.checkDisconnected(transport.DISCONNECT_KEY_EXCHANGE_FAILED)
 
 
+    def test_disconnectKEX_ECDH_REPLYBadSignature(self):
+        """
+        Test that KEX_ECDH_REPLY disconnects if the signature is bad.
+        """
+        kexmsg = (
+            b"\xAA" * 16 +
+            common.NS(b'ecdh-sha2-nistp256') +
+            common.NS(b'ssh-rsa') +
+            common.NS(b'aes256-ctr') +
+            common.NS(b'aes256-ctr') +
+            common.NS(b'hmac-sha1') +
+            common.NS(b'hmac-sha1') +
+            common.NS(b'none') +
+            common.NS(b'none') +
+            common.NS(b'') +
+            common.NS(b'') +
+            b'\x00' + b'\x00\x00\x00\x00')
+
+        self.proto.ssh_KEXINIT(kexmsg)
+
+        self.proto.dataReceived(b"SSH-2.0-OpenSSH\r\n")
+
+        self.proto.ecPriv = ec.generate_private_key(ec.SECP256R1(),
+                                                    default_backend())
+        self.proto.ecPub = self.proto.ecPriv.public_key()
+
+        # Generate the private key
+        thisPriv = ec.generate_private_key(ec.SECP256R1(), default_backend())
+        # Get the public key
+        thisPub = thisPriv.public_key()
+        encPub = thisPub.public_numbers().encode_point()
+        self.proto.curve = ec.SECP256R1()
+
+        self.proto.kexAlg = b'ecdh-sha2-nistp256'
+
+        self.proto._ssh_KEX_ECDH_REPLY(
+             common.NS(MockFactory().getPublicKeys()[b'ssh-rsa'].blob()) +
+                       common.NS(encPub) + common.NS(b'bad-signature'))
+
+        self.checkDisconnected(transport.DISCONNECT_KEY_EXCHANGE_FAILED)
+
+
     def test_disconnectNEWKEYSData(self):
         """
         Test that NEWKEYS disconnects if it receives data.
@@ -1963,8 +2087,8 @@ class ClientSSHTransportDHGroupExchangeBaseCase(ClientSSHTransportBaseCase):
         exchangeHash = h.digest()
 
         def _cbTestKEX_DH_GEX_REPLY(value):
-            self.assertIs(value, None)
-            self.assertEqual(self.calledVerifyHostKey, True)
+            self.assertIsNone(value)
+            self.assertTrue(self.calledVerifyHostKey)
             self.assertEqual(self.proto.sessionID, exchangeHash)
 
         signature = self.privObj.sign(exchangeHash)
@@ -1986,6 +2110,48 @@ class ClientSSHTransportDHGroupExchangeBaseCase(ClientSSHTransportBaseCase):
         self.checkDisconnected(transport.DISCONNECT_KEY_EXCHANGE_FAILED)
 
 
+    def test_disconnectKEX_ECDH_REPLYBadSignature(self):
+        """
+        Test that KEX_ECDH_REPLY disconnects if the signature is bad.
+        """
+        kexmsg = (
+            b"\xAA" * 16 +
+            common.NS(b'ecdh-sha2-nistp256') +
+            common.NS(b'ssh-rsa') +
+            common.NS(b'aes256-ctr') +
+            common.NS(b'aes256-ctr') +
+            common.NS(b'hmac-sha1') +
+            common.NS(b'hmac-sha1') +
+            common.NS(b'none') +
+            common.NS(b'none') +
+            common.NS(b'') +
+            common.NS(b'') +
+            b'\x00' + b'\x00\x00\x00\x00')
+
+        self.proto.ssh_KEXINIT(kexmsg)
+
+        self.proto.dataReceived(b"SSH-2.0-OpenSSH\r\n")
+
+        self.proto.ecPriv = ec.generate_private_key(ec.SECP256R1(),
+                                                    default_backend())
+        self.proto.ecPub = self.proto.ecPriv.public_key()
+
+        # Generate the private key
+        thisPriv = ec.generate_private_key(ec.SECP256R1(), default_backend())
+        # Get the public key
+        thisPub = thisPriv.public_key()
+        encPub = thisPub.public_numbers().encode_point()
+        self.proto.curve = ec.SECP256R1()
+
+        self.proto.kexAlg = b'ecdh-sha2-nistp256'
+
+        self.proto._ssh_KEX_ECDH_REPLY(
+            common.NS(MockFactory().getPublicKeys()[b'ssh-rsa'].blob()) +
+            common.NS(encPub) + common.NS(b'bad-signature'))
+
+        self.checkDisconnected(transport.DISCONNECT_KEY_EXCHANGE_FAILED)
+
+
 
 class ClientSSHTransportDHGroupExchangeSHA1Tests(
         ClientSSHTransportDHGroupExchangeBaseCase, DHGroupExchangeSHA1Mixin,
@@ -2022,7 +2188,7 @@ class GetMACTests(unittest.TestCase):
         Generate a new shared secret to be used with the tests.
 
         @return: A new secret.
-        @rtype: C{bytes}
+        @rtype: L{bytes}
         """
         return insecureRandom(64)
 
@@ -2035,25 +2201,25 @@ class GetMACTests(unittest.TestCase):
         attribute set to the value of the key supplied.
 
         @param hmacName: Identifier of HMAC algorithm.
-        @type hmacName: C{bytes}
+        @type hmacName: L{bytes}
 
         @param hashProcessor: Callable for the hash algorithm.
         @type hashProcessor: C{callable}
 
         @param digestSize: Size of the digest for algorithm.
-        @type digestSize: C{int}
+        @type digestSize: L{int}
 
         @param blockPadSize: Size of padding applied to the shared secret to
             match the block size.
-        @type blockPadSize: C{int}
+        @type blockPadSize: L{int}
         """
         secret = self.getSharedSecret()
 
         params = self.ciphers._getMAC(hmacName, secret)
 
         key = secret[:digestSize] + b'\x00' * blockPadSize
-        innerPad = b''.join(chr(ord(b) ^ 0x36) for b in key)
-        outerPad = b''.join(chr(ord(b) ^ 0x5c) for b in key)
+        innerPad = b''.join(chr(ord(b) ^ 0x36) for b in iterbytes(key))
+        outerPad = b''.join(chr(ord(b) ^ 0x5c) for b in iterbytes(key))
         self.assertEqual(
             (hashProcessor, innerPad, outerPad, digestSize), params)
         self.assertEqual(key, params.key)
@@ -2070,6 +2236,17 @@ class GetMACTests(unittest.TestCase):
             b"hmac-sha2-512", sha512, digestSize=64, blockPadSize=64)
 
 
+    def test_hmacsha2384(self):
+        """
+        When L{SSHCiphers._getMAC} is called with the C{b"hmac-sha2-384"} MAC
+        algorithm name it returns a tuple of (sha384 digest object, inner pad,
+        outer pad, sha384 digest size) with a C{key} attribute set to the
+        value of the key supplied.
+        """
+        self.assertGetMAC(
+            b"hmac-sha2-384", sha384, digestSize=48, blockPadSize=80)
+
+
     def test_hmacsha2256(self):
         """
         When L{SSHCiphers._getMAC} is called with the C{b"hmac-sha2-256"} MAC
@@ -2258,6 +2435,8 @@ class TransportLoopbackTests(unittest.TestCase):
         client.receiveError = lambda code, desc: client.errors.append((
                 code, desc))
         client.connectionSecure = lambda: client.loseConnection()
+        server.supportedPublicKeys = list(
+                server.factory.getPublicKeys().keys())
         server = mod(server)
         client = mod(client)
         def check(ignored, server, client):
@@ -2355,10 +2534,10 @@ class RandomNumberTests(unittest.TestCase):
         L{_getRandomNumber} returns an integer constructed directly from the
         bytes returned by the random byte generator passed to it.
         """
-        def random(bytes):
+        def random(data):
             # The number of bytes requested will be the value of each byte
             # we return.
-            return chr(bytes) * bytes
+            return chr(data) * data
         self.assertEqual(
             transport._getRandomNumber(random, 32),
             4 << 24 | 4 << 16 | 4 << 8 | 4)
@@ -2381,8 +2560,8 @@ class RandomNumberTests(unittest.TestCase):
         discarded and another attempt is made to produce a larger value.
         """
         results = [chr(0), chr(1), chr(127)]
-        def random(bytes):
-            return results.pop(0) * bytes
+        def random(data):
+            return results.pop(0) * data
         self.assertEqual(
             transport._generateX(random, 8),
             127)
@@ -2396,8 +2575,8 @@ class RandomNumberTests(unittest.TestCase):
         value.
         """
         results = [chr(255), chr(64)]
-        def random(bytes):
-            return results.pop(0) * bytes
+        def random(data):
+            return results.pop(0) * data
         self.assertEqual(
             transport._generateX(random, 8),
             64)
